/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
#ifndef nsHtml5DocumentMode_h
#define nsHtml5DocumentMode_h

enum nsHtml5DocumentMode {
    STANDARDS_MODE,
    ALMOST_STANDARDS_MODE,
    QUIRKS_MODE
};

#endif // nsHtml5DocumentMode_h
