/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

/**
 * nsIPrintSession
 *
 * Stores data pertaining only to a single print job. This
 * differs from nsIPrintSettings, which stores data which may
 * be valid across a number of jobs.
 *
 * The creation of a component which implements this interface
 * will begin the session. Likewise, destruction of that object
 * will end the session.
 * 
 * @status
 */

%{ C++
namespace mozilla {
namespace layout {
class RemotePrintJobChild;
}
}
%}

[ptr] native RemotePrintJobChildPtr(mozilla::layout::RemotePrintJobChild);

[uuid(424ae4bb-10ca-4f35-b84e-eab893322df4)]

interface nsIPrintSession : nsISupports
{
  /**
   * The remote print job is used for printing via the parent process.
   */
  [noscript] attribute RemotePrintJobChildPtr remotePrintJob;
};
