#define MOZ_SOURCE_STAMP a5c63e3e18c9231beb1a455b611e5a8bd77b764e
#define MOZ_SOURCE_REPO https://hg.mozilla.org/releases/mozilla-esr52
#define MOZ_SOURCE_URL https://hg.mozilla.org/releases/mozilla-esr52/rev/a5c63e3e18c9231beb1a455b611e5a8bd77b764e
