/*
 * Copyright (c) 2008-2010 Mozilla Foundation
 *
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS IN THE SOFTWARE.
 */

HTML5_ATOM(emptystring, "")
HTML5_ATOM(title, "title")
HTML5_ATOM(desc, "desc")
HTML5_ATOM(foreignObject, "foreignObject")
HTML5_ATOM(mi, "mi")
HTML5_ATOM(mo, "mo")
HTML5_ATOM(mn, "mn")
HTML5_ATOM(ms, "ms")
HTML5_ATOM(mtext, "mtext")
HTML5_ATOM(annotation_xml, "annotation-xml")
HTML5_ATOM(template_, "template")
HTML5_ATOM(textarea, "textarea")
HTML5_ATOM(style, "style")
HTML5_ATOM(xmp, "xmp")
HTML5_ATOM(iframe, "iframe")
HTML5_ATOM(noembed, "noembed")
HTML5_ATOM(noframes, "noframes")
HTML5_ATOM(noscript, "noscript")
HTML5_ATOM(plaintext, "plaintext")
HTML5_ATOM(script, "script")
HTML5_ATOM(svg, "svg")
HTML5_ATOM(table, "table")
HTML5_ATOM(caption, "caption")
HTML5_ATOM(p, "p")
HTML5_ATOM(address, "address")
HTML5_ATOM(div, "div")
HTML5_ATOM(a, "a")
HTML5_ATOM(nobr, "nobr")
HTML5_ATOM(input, "input")
HTML5_ATOM(option, "option")
HTML5_ATOM(ruby, "ruby")
HTML5_ATOM(rtc, "rtc")
HTML5_ATOM(select, "select")
HTML5_ATOM(optgroup, "optgroup")
HTML5_ATOM(tbody, "tbody")
HTML5_ATOM(tfoot, "tfoot")
HTML5_ATOM(thead, "thead")
HTML5_ATOM(frameset, "frameset")
HTML5_ATOM(button, "button")
HTML5_ATOM(ul, "ul")
HTML5_ATOM(ol, "ol")
HTML5_ATOM(html, "html")
HTML5_ATOM(td, "td")
HTML5_ATOM(th, "th")
HTML5_ATOM(tr, "tr")
HTML5_ATOM(colgroup, "colgroup")
HTML5_ATOM(head, "head")
HTML5_ATOM(body, "body")
HTML5_ATOM(form, "form")
HTML5_ATOM(xmlns, "xmlns")
HTML5_ATOM(xlink, "xlink")
HTML5_ATOM(xml, "xml")
HTML5_ATOM(d, "d")
HTML5_ATOM(k, "k")
HTML5_ATOM(r, "r")
HTML5_ATOM(x, "x")
HTML5_ATOM(y, "y")
HTML5_ATOM(z, "z")
HTML5_ATOM(by, "by")
HTML5_ATOM(cx, "cx")
HTML5_ATOM(cy, "cy")
HTML5_ATOM(dx, "dx")
HTML5_ATOM(dy, "dy")
HTML5_ATOM(g2, "g2")
HTML5_ATOM(g1, "g1")
HTML5_ATOM(fx, "fx")
HTML5_ATOM(fy, "fy")
HTML5_ATOM(k4, "k4")
HTML5_ATOM(k2, "k2")
HTML5_ATOM(k3, "k3")
HTML5_ATOM(k1, "k1")
HTML5_ATOM(id, "id")
HTML5_ATOM(in, "in")
HTML5_ATOM(u2, "u2")
HTML5_ATOM(u1, "u1")
HTML5_ATOM(rt, "rt")
HTML5_ATOM(rx, "rx")
HTML5_ATOM(ry, "ry")
HTML5_ATOM(to, "to")
HTML5_ATOM(y2, "y2")
HTML5_ATOM(y1, "y1")
HTML5_ATOM(x1, "x1")
HTML5_ATOM(x2, "x2")
HTML5_ATOM(alt, "alt")
HTML5_ATOM(dir, "dir")
HTML5_ATOM(dur, "dur")
HTML5_ATOM(end, "end")
HTML5_ATOM(for_, "for")
HTML5_ATOM(in2, "in2")
HTML5_ATOM(max, "max")
HTML5_ATOM(min, "min")
HTML5_ATOM(low, "low")
HTML5_ATOM(rel, "rel")
HTML5_ATOM(rev, "rev")
HTML5_ATOM(src, "src")
HTML5_ATOM(axis, "axis")
HTML5_ATOM(abbr, "abbr")
HTML5_ATOM(bbox, "bbox")
HTML5_ATOM(cite, "cite")
HTML5_ATOM(code, "code")
HTML5_ATOM(bias, "bias")
HTML5_ATOM(cols, "cols")
HTML5_ATOM(clip, "clip")
HTML5_ATOM(char_, "char")
HTML5_ATOM(base, "base")
HTML5_ATOM(edge, "edge")
HTML5_ATOM(data, "data")
HTML5_ATOM(fill, "fill")
HTML5_ATOM(from, "from")
HTML5_ATOM(face, "face")
HTML5_ATOM(high, "high")
HTML5_ATOM(href, "href")
HTML5_ATOM(open, "open")
HTML5_ATOM(icon, "icon")
HTML5_ATOM(name, "name")
HTML5_ATOM(mode, "mode")
HTML5_ATOM(mask, "mask")
HTML5_ATOM(link, "link")
HTML5_ATOM(lang, "lang")
HTML5_ATOM(loop, "loop")
HTML5_ATOM(list, "list")
HTML5_ATOM(type, "type")
HTML5_ATOM(when, "when")
HTML5_ATOM(wrap, "wrap")
HTML5_ATOM(text, "text")
HTML5_ATOM(path, "path")
HTML5_ATOM(ping, "ping")
HTML5_ATOM(refx, "refx")
HTML5_ATOM(refX, "refX")
HTML5_ATOM(refy, "refy")
HTML5_ATOM(refY, "refY")
HTML5_ATOM(size, "size")
HTML5_ATOM(seed, "seed")
HTML5_ATOM(rows, "rows")
HTML5_ATOM(span, "span")
HTML5_ATOM(step, "step")
HTML5_ATOM(role, "role")
HTML5_ATOM(xref, "xref")
HTML5_ATOM(async, "async")
HTML5_ATOM(alink, "alink")
HTML5_ATOM(align, "align")
HTML5_ATOM(close, "close")
HTML5_ATOM(color, "color")
HTML5_ATOM(class_, "class")
HTML5_ATOM(clear, "clear")
HTML5_ATOM(begin, "begin")
HTML5_ATOM(depth, "depth")
HTML5_ATOM(defer, "defer")
HTML5_ATOM(fence, "fence")
HTML5_ATOM(frame, "frame")
HTML5_ATOM(ismap, "ismap")
HTML5_ATOM(onend, "onend")
HTML5_ATOM(index, "index")
HTML5_ATOM(order, "order")
HTML5_ATOM(other, "other")
HTML5_ATOM(oncut, "oncut")
HTML5_ATOM(nargs, "nargs")
HTML5_ATOM(media, "media")
HTML5_ATOM(label, "label")
HTML5_ATOM(local, "local")
HTML5_ATOM(width, "width")
HTML5_ATOM(vlink, "vlink")
HTML5_ATOM(value, "value")
HTML5_ATOM(slope, "slope")
HTML5_ATOM(shape, "shape")
HTML5_ATOM(scope, "scope")
HTML5_ATOM(scale, "scale")
HTML5_ATOM(speed, "speed")
HTML5_ATOM(rules, "rules")
HTML5_ATOM(stemh, "stemh")
HTML5_ATOM(sizes, "sizes")
HTML5_ATOM(stemv, "stemv")
HTML5_ATOM(start, "start")
HTML5_ATOM(accept, "accept")
HTML5_ATOM(accent, "accent")
HTML5_ATOM(ascent, "ascent")
HTML5_ATOM(active, "active")
HTML5_ATOM(altimg, "altimg")
HTML5_ATOM(action, "action")
HTML5_ATOM(border, "border")
HTML5_ATOM(cursor, "cursor")
HTML5_ATOM(coords, "coords")
HTML5_ATOM(filter, "filter")
HTML5_ATOM(format, "format")
HTML5_ATOM(hidden, "hidden")
HTML5_ATOM(hspace, "hspace")
HTML5_ATOM(height, "height")
HTML5_ATOM(onmove, "onmove")
HTML5_ATOM(onload, "onload")
HTML5_ATOM(ondrag, "ondrag")
HTML5_ATOM(origin, "origin")
HTML5_ATOM(onzoom, "onzoom")
HTML5_ATOM(onhelp, "onhelp")
HTML5_ATOM(onstop, "onstop")
HTML5_ATOM(ondrop, "ondrop")
HTML5_ATOM(onblur, "onblur")
HTML5_ATOM(object, "object")
HTML5_ATOM(offset, "offset")
HTML5_ATOM(orient, "orient")
HTML5_ATOM(oncopy, "oncopy")
HTML5_ATOM(nowrap, "nowrap")
HTML5_ATOM(nohref, "nohref")
HTML5_ATOM(macros, "macros")
HTML5_ATOM(method, "method")
HTML5_ATOM(lowsrc, "lowsrc")
HTML5_ATOM(lspace, "lspace")
HTML5_ATOM(lquote, "lquote")
HTML5_ATOM(usemap, "usemap")
HTML5_ATOM(widths, "widths")
HTML5_ATOM(target, "target")
HTML5_ATOM(values, "values")
HTML5_ATOM(valign, "valign")
HTML5_ATOM(vspace, "vspace")
HTML5_ATOM(poster, "poster")
HTML5_ATOM(points, "points")
HTML5_ATOM(prompt, "prompt")
HTML5_ATOM(srcdoc, "srcdoc")
HTML5_ATOM(scoped, "scoped")
HTML5_ATOM(string, "string")
HTML5_ATOM(scheme, "scheme")
HTML5_ATOM(stroke, "stroke")
HTML5_ATOM(radius, "radius")
HTML5_ATOM(result, "result")
HTML5_ATOM(repeat, "repeat")
HTML5_ATOM(srcset, "srcset")
HTML5_ATOM(rspace, "rspace")
HTML5_ATOM(rotate, "rotate")
HTML5_ATOM(rquote, "rquote")
HTML5_ATOM(alttext, "alttext")
HTML5_ATOM(archive, "archive")
HTML5_ATOM(azimuth, "azimuth")
HTML5_ATOM(closure, "closure")
HTML5_ATOM(checked, "checked")
HTML5_ATOM(classid, "classid")
HTML5_ATOM(charoff, "charoff")
HTML5_ATOM(bgcolor, "bgcolor")
HTML5_ATOM(colspan, "colspan")
HTML5_ATOM(charset, "charset")
HTML5_ATOM(compact, "compact")
HTML5_ATOM(content, "content")
HTML5_ATOM(enctype, "enctype")
HTML5_ATOM(datasrc, "datasrc")
HTML5_ATOM(datafld, "datafld")
HTML5_ATOM(declare, "declare")
HTML5_ATOM(display, "display")
HTML5_ATOM(divisor, "divisor")
HTML5_ATOM(default_, "default")
HTML5_ATOM(descent, "descent")
HTML5_ATOM(kerning, "kerning")
HTML5_ATOM(hanging, "hanging")
HTML5_ATOM(headers, "headers")
HTML5_ATOM(onpaste, "onpaste")
HTML5_ATOM(onclick, "onclick")
HTML5_ATOM(optimum, "optimum")
HTML5_ATOM(onbegin, "onbegin")
HTML5_ATOM(onkeyup, "onkeyup")
HTML5_ATOM(onfocus, "onfocus")
HTML5_ATOM(onerror, "onerror")
HTML5_ATOM(oninput, "oninput")
HTML5_ATOM(onabort, "onabort")
HTML5_ATOM(onstart, "onstart")
HTML5_ATOM(onreset, "onreset")
HTML5_ATOM(opacity, "opacity")
HTML5_ATOM(noshade, "noshade")
HTML5_ATOM(minsize, "minsize")
HTML5_ATOM(maxsize, "maxsize")
HTML5_ATOM(largeop, "largeop")
HTML5_ATOM(unicode_, "unicode")
HTML5_ATOM(targetx, "targetx")
HTML5_ATOM(targetX, "targetX")
HTML5_ATOM(targety, "targety")
HTML5_ATOM(targetY, "targetY")
HTML5_ATOM(viewbox, "viewbox")
HTML5_ATOM(viewBox, "viewBox")
HTML5_ATOM(version, "version")
HTML5_ATOM(pattern, "pattern")
HTML5_ATOM(profile, "profile")
HTML5_ATOM(spacing, "spacing")
HTML5_ATOM(restart, "restart")
HTML5_ATOM(rowspan, "rowspan")
HTML5_ATOM(sandbox, "sandbox")
HTML5_ATOM(summary, "summary")
HTML5_ATOM(standby, "standby")
HTML5_ATOM(replace, "replace")
HTML5_ATOM(autoplay, "autoplay")
HTML5_ATOM(additive, "additive")
HTML5_ATOM(calcmode, "calcmode")
HTML5_ATOM(calcMode, "calcMode")
HTML5_ATOM(codetype, "codetype")
HTML5_ATOM(codebase, "codebase")
HTML5_ATOM(controls, "controls")
HTML5_ATOM(bevelled, "bevelled")
HTML5_ATOM(baseline, "baseline")
HTML5_ATOM(exponent, "exponent")
HTML5_ATOM(edgemode, "edgemode")
HTML5_ATOM(edgeMode, "edgeMode")
HTML5_ATOM(encoding, "encoding")
HTML5_ATOM(glyphref, "glyphref")
HTML5_ATOM(glyphRef, "glyphRef")
HTML5_ATOM(datetime, "datetime")
HTML5_ATOM(disabled, "disabled")
HTML5_ATOM(fontsize, "fontsize")
HTML5_ATOM(keytimes, "keytimes")
HTML5_ATOM(keyTimes, "keyTimes")
HTML5_ATOM(panose_1, "panose-1")
HTML5_ATOM(hreflang, "hreflang")
HTML5_ATOM(onresize, "onresize")
HTML5_ATOM(onchange, "onchange")
HTML5_ATOM(onbounce, "onbounce")
HTML5_ATOM(onunload, "onunload")
HTML5_ATOM(onfinish, "onfinish")
HTML5_ATOM(onscroll, "onscroll")
HTML5_ATOM(operator_, "operator")
HTML5_ATOM(overflow, "overflow")
HTML5_ATOM(onsubmit, "onsubmit")
HTML5_ATOM(onrepeat, "onrepeat")
HTML5_ATOM(onselect, "onselect")
HTML5_ATOM(notation, "notation")
HTML5_ATOM(noresize, "noresize")
HTML5_ATOM(manifest, "manifest")
HTML5_ATOM(mathsize, "mathsize")
HTML5_ATOM(multiple, "multiple")
HTML5_ATOM(longdesc, "longdesc")
HTML5_ATOM(language, "language")
HTML5_ATOM(tabindex, "tabindex")
HTML5_ATOM(property, "property")
HTML5_ATOM(readonly, "readonly")
HTML5_ATOM(selected, "selected")
HTML5_ATOM(rowlines, "rowlines")
HTML5_ATOM(seamless, "seamless")
HTML5_ATOM(rowalign, "rowalign")
HTML5_ATOM(stretchy, "stretchy")
HTML5_ATOM(required, "required")
HTML5_ATOM(xml_base, "xml:base")
HTML5_ATOM(xml_lang, "xml:lang")
HTML5_ATOM(x_height, "x-height")
HTML5_ATOM(aria_owns, "aria-owns")
HTML5_ATOM(autofocus, "autofocus")
HTML5_ATOM(aria_sort, "aria-sort")
HTML5_ATOM(accesskey, "accesskey")
HTML5_ATOM(aria_busy, "aria-busy")
HTML5_ATOM(aria_grab, "aria-grab")
HTML5_ATOM(amplitude, "amplitude")
HTML5_ATOM(aria_live, "aria-live")
HTML5_ATOM(clip_rule, "clip-rule")
HTML5_ATOM(clip_path, "clip-path")
HTML5_ATOM(equalrows, "equalrows")
HTML5_ATOM(elevation, "elevation")
HTML5_ATOM(direction, "direction")
HTML5_ATOM(draggable, "draggable")
HTML5_ATOM(fill_rule, "fill-rule")
HTML5_ATOM(fontstyle, "fontstyle")
HTML5_ATOM(font_size, "font-size")
HTML5_ATOM(keysystem, "keysystem")
HTML5_ATOM(keypoints, "keypoints")
HTML5_ATOM(keyPoints, "keyPoints")
HTML5_ATOM(hidefocus, "hidefocus")
HTML5_ATOM(onmessage, "onmessage")
HTML5_ATOM(intercept, "intercept")
HTML5_ATOM(ondragend, "ondragend")
HTML5_ATOM(onmoveend, "onmoveend")
HTML5_ATOM(oninvalid, "oninvalid")
HTML5_ATOM(integrity, "integrity")
HTML5_ATOM(onkeydown, "onkeydown")
HTML5_ATOM(onfocusin, "onfocusin")
HTML5_ATOM(onmouseup, "onmouseup")
HTML5_ATOM(inputmode, "inputmode")
HTML5_ATOM(onrowexit, "onrowexit")
HTML5_ATOM(mathcolor, "mathcolor")
HTML5_ATOM(maskunits, "maskunits")
HTML5_ATOM(maskUnits, "maskUnits")
HTML5_ATOM(maxlength, "maxlength")
HTML5_ATOM(linebreak, "linebreak")
HTML5_ATOM(transform, "transform")
HTML5_ATOM(v_hanging, "v-hanging")
HTML5_ATOM(valuetype, "valuetype")
HTML5_ATOM(pointsatz, "pointsatz")
HTML5_ATOM(pointsAtZ, "pointsAtZ")
HTML5_ATOM(pointsatx, "pointsatx")
HTML5_ATOM(pointsAtX, "pointsAtX")
HTML5_ATOM(pointsaty, "pointsaty")
HTML5_ATOM(pointsAtY, "pointsAtY")
HTML5_ATOM(symmetric, "symmetric")
HTML5_ATOM(scrolling, "scrolling")
HTML5_ATOM(repeatdur, "repeatdur")
HTML5_ATOM(repeatDur, "repeatDur")
HTML5_ATOM(selection, "selection")
HTML5_ATOM(separator, "separator")
HTML5_ATOM(xml_space, "xml:space")
HTML5_ATOM(space, "space")
HTML5_ATOM(autosubmit, "autosubmit")
HTML5_ATOM(alphabetic, "alphabetic")
HTML5_ATOM(actiontype, "actiontype")
HTML5_ATOM(accumulate, "accumulate")
HTML5_ATOM(aria_level, "aria-level")
HTML5_ATOM(columnspan, "columnspan")
HTML5_ATOM(cap_height, "cap-height")
HTML5_ATOM(background, "background")
HTML5_ATOM(glyph_name, "glyph-name")
HTML5_ATOM(groupalign, "groupalign")
HTML5_ATOM(fontfamily, "fontfamily")
HTML5_ATOM(fontweight, "fontweight")
HTML5_ATOM(font_style, "font-style")
HTML5_ATOM(keysplines, "keysplines")
HTML5_ATOM(keySplines, "keySplines")
HTML5_ATOM(http_equiv, "http-equiv")
HTML5_ATOM(onactivate, "onactivate")
HTML5_ATOM(occurrence, "occurrence")
HTML5_ATOM(irrelevant, "irrelevant")
HTML5_ATOM(ondblclick, "ondblclick")
HTML5_ATOM(ondragdrop, "ondragdrop")
HTML5_ATOM(onkeypress, "onkeypress")
HTML5_ATOM(onrowenter, "onrowenter")
HTML5_ATOM(ondragover, "ondragover")
HTML5_ATOM(onfocusout, "onfocusout")
HTML5_ATOM(onmouseout, "onmouseout")
HTML5_ATOM(numoctaves, "numoctaves")
HTML5_ATOM(numOctaves, "numOctaves")
HTML5_ATOM(marker_mid, "marker-mid")
HTML5_ATOM(marker_end, "marker-end")
HTML5_ATOM(textlength, "textlength")
HTML5_ATOM(textLength, "textLength")
HTML5_ATOM(visibility, "visibility")
HTML5_ATOM(viewtarget, "viewtarget")
HTML5_ATOM(viewTarget, "viewTarget")
HTML5_ATOM(vert_adv_y, "vert-adv-y")
HTML5_ATOM(pathlength, "pathlength")
HTML5_ATOM(pathLength, "pathLength")
HTML5_ATOM(repeat_max, "repeat-max")
HTML5_ATOM(radiogroup, "radiogroup")
HTML5_ATOM(stop_color, "stop-color")
HTML5_ATOM(separators, "separators")
HTML5_ATOM(repeat_min, "repeat-min")
HTML5_ATOM(rowspacing, "rowspacing")
HTML5_ATOM(zoomandpan, "zoomandpan")
HTML5_ATOM(zoomAndPan, "zoomAndPan")
HTML5_ATOM(xlink_type, "xlink:type")
HTML5_ATOM(xlink_role, "xlink:role")
HTML5_ATOM(xlink_href, "xlink:href")
HTML5_ATOM(xlink_show, "xlink:show")
HTML5_ATOM(show, "show")
HTML5_ATOM(accentunder, "accentunder")
HTML5_ATOM(aria_secret, "aria-secret")
HTML5_ATOM(aria_atomic, "aria-atomic")
HTML5_ATOM(aria_hidden, "aria-hidden")
HTML5_ATOM(aria_flowto, "aria-flowto")
HTML5_ATOM(arabic_form, "arabic-form")
HTML5_ATOM(cellpadding, "cellpadding")
HTML5_ATOM(cellspacing, "cellspacing")
HTML5_ATOM(columnwidth, "columnwidth")
HTML5_ATOM(crossorigin, "crossorigin")
HTML5_ATOM(columnalign, "columnalign")
HTML5_ATOM(columnlines, "columnlines")
HTML5_ATOM(contextmenu, "contextmenu")
HTML5_ATOM(baseprofile, "baseprofile")
HTML5_ATOM(baseProfile, "baseProfile")
HTML5_ATOM(font_family, "font-family")
HTML5_ATOM(frameborder, "frameborder")
HTML5_ATOM(filterunits, "filterunits")
HTML5_ATOM(filterUnits, "filterUnits")
HTML5_ATOM(flood_color, "flood-color")
HTML5_ATOM(font_weight, "font-weight")
HTML5_ATOM(horiz_adv_x, "horiz-adv-x")
HTML5_ATOM(ondragleave, "ondragleave")
HTML5_ATOM(onmousemove, "onmousemove")
HTML5_ATOM(orientation, "orientation")
HTML5_ATOM(onmousedown, "onmousedown")
HTML5_ATOM(onmouseover, "onmouseover")
HTML5_ATOM(ondragenter, "ondragenter")
HTML5_ATOM(ideographic, "ideographic")
HTML5_ATOM(onbeforecut, "onbeforecut")
HTML5_ATOM(onforminput, "onforminput")
HTML5_ATOM(ondragstart, "ondragstart")
HTML5_ATOM(onmovestart, "onmovestart")
HTML5_ATOM(markerunits, "markerunits")
HTML5_ATOM(markerUnits, "markerUnits")
HTML5_ATOM(mathvariant, "mathvariant")
HTML5_ATOM(marginwidth, "marginwidth")
HTML5_ATOM(markerwidth, "markerwidth")
HTML5_ATOM(markerWidth, "markerWidth")
HTML5_ATOM(text_anchor, "text-anchor")
HTML5_ATOM(tablevalues, "tablevalues")
HTML5_ATOM(tableValues, "tableValues")
HTML5_ATOM(scriptlevel, "scriptlevel")
HTML5_ATOM(repeatcount, "repeatcount")
HTML5_ATOM(repeatCount, "repeatCount")
HTML5_ATOM(stitchtiles, "stitchtiles")
HTML5_ATOM(stitchTiles, "stitchTiles")
HTML5_ATOM(startoffset, "startoffset")
HTML5_ATOM(startOffset, "startOffset")
HTML5_ATOM(scrolldelay, "scrolldelay")
HTML5_ATOM(xmlns_xlink, "xmlns:xlink")
HTML5_ATOM(xlink_title, "xlink:title")
HTML5_ATOM(aria_invalid, "aria-invalid")
HTML5_ATOM(aria_pressed, "aria-pressed")
HTML5_ATOM(aria_checked, "aria-checked")
HTML5_ATOM(autocomplete, "autocomplete")
HTML5_ATOM(aria_setsize, "aria-setsize")
HTML5_ATOM(aria_channel, "aria-channel")
HTML5_ATOM(equalcolumns, "equalcolumns")
HTML5_ATOM(displaystyle, "displaystyle")
HTML5_ATOM(dataformatas, "dataformatas")
HTML5_ATOM(fill_opacity, "fill-opacity")
HTML5_ATOM(font_variant, "font-variant")
HTML5_ATOM(font_stretch, "font-stretch")
HTML5_ATOM(framespacing, "framespacing")
HTML5_ATOM(kernelmatrix, "kernelmatrix")
HTML5_ATOM(kernelMatrix, "kernelMatrix")
HTML5_ATOM(ondeactivate, "ondeactivate")
HTML5_ATOM(onrowsdelete, "onrowsdelete")
HTML5_ATOM(onmouseleave, "onmouseleave")
HTML5_ATOM(onformchange, "onformchange")
HTML5_ATOM(oncellchange, "oncellchange")
HTML5_ATOM(onmousewheel, "onmousewheel")
HTML5_ATOM(onmouseenter, "onmouseenter")
HTML5_ATOM(onafterprint, "onafterprint")
HTML5_ATOM(onbeforecopy, "onbeforecopy")
HTML5_ATOM(marginheight, "marginheight")
HTML5_ATOM(markerheight, "markerheight")
HTML5_ATOM(markerHeight, "markerHeight")
HTML5_ATOM(marker_start, "marker-start")
HTML5_ATOM(mathematical, "mathematical")
HTML5_ATOM(lengthadjust, "lengthadjust")
HTML5_ATOM(lengthAdjust, "lengthAdjust")
HTML5_ATOM(unselectable, "unselectable")
HTML5_ATOM(unicode_bidi, "unicode-bidi")
HTML5_ATOM(units_per_em, "units-per-em")
HTML5_ATOM(word_spacing, "word-spacing")
HTML5_ATOM(writing_mode, "writing-mode")
HTML5_ATOM(v_alphabetic, "v-alphabetic")
HTML5_ATOM(patternunits, "patternunits")
HTML5_ATOM(patternUnits, "patternUnits")
HTML5_ATOM(spreadmethod, "spreadmethod")
HTML5_ATOM(spreadMethod, "spreadMethod")
HTML5_ATOM(surfacescale, "surfacescale")
HTML5_ATOM(surfaceScale, "surfaceScale")
HTML5_ATOM(stroke_width, "stroke-width")
HTML5_ATOM(repeat_start, "repeat-start")
HTML5_ATOM(stddeviation, "stddeviation")
HTML5_ATOM(stdDeviation, "stdDeviation")
HTML5_ATOM(stop_opacity, "stop-opacity")
HTML5_ATOM(aria_controls, "aria-controls")
HTML5_ATOM(aria_haspopup, "aria-haspopup")
HTML5_ATOM(accent_height, "accent-height")
HTML5_ATOM(aria_valuenow, "aria-valuenow")
HTML5_ATOM(aria_relevant, "aria-relevant")
HTML5_ATOM(aria_posinset, "aria-posinset")
HTML5_ATOM(aria_valuemax, "aria-valuemax")
HTML5_ATOM(aria_readonly, "aria-readonly")
HTML5_ATOM(aria_selected, "aria-selected")
HTML5_ATOM(aria_required, "aria-required")
HTML5_ATOM(aria_expanded, "aria-expanded")
HTML5_ATOM(aria_disabled, "aria-disabled")
HTML5_ATOM(attributetype, "attributetype")
HTML5_ATOM(attributeType, "attributeType")
HTML5_ATOM(attributename, "attributename")
HTML5_ATOM(attributeName, "attributeName")
HTML5_ATOM(aria_datatype, "aria-datatype")
HTML5_ATOM(aria_valuemin, "aria-valuemin")
HTML5_ATOM(basefrequency, "basefrequency")
HTML5_ATOM(baseFrequency, "baseFrequency")
HTML5_ATOM(columnspacing, "columnspacing")
HTML5_ATOM(color_profile, "color-profile")
HTML5_ATOM(clippathunits, "clippathunits")
HTML5_ATOM(clipPathUnits, "clipPathUnits")
HTML5_ATOM(definitionurl, "definitionurl")
HTML5_ATOM(definitionURL, "definitionURL")
HTML5_ATOM(gradientunits, "gradientunits")
HTML5_ATOM(gradientUnits, "gradientUnits")
HTML5_ATOM(flood_opacity, "flood-opacity")
HTML5_ATOM(onafterupdate, "onafterupdate")
HTML5_ATOM(onerrorupdate, "onerrorupdate")
HTML5_ATOM(onbeforepaste, "onbeforepaste")
HTML5_ATOM(onlosecapture, "onlosecapture")
HTML5_ATOM(oncontextmenu, "oncontextmenu")
HTML5_ATOM(onselectstart, "onselectstart")
HTML5_ATOM(onbeforeprint, "onbeforeprint")
HTML5_ATOM(movablelimits, "movablelimits")
HTML5_ATOM(linethickness, "linethickness")
HTML5_ATOM(unicode_range, "unicode-range")
HTML5_ATOM(thinmathspace, "thinmathspace")
HTML5_ATOM(vert_origin_x, "vert-origin-x")
HTML5_ATOM(vert_origin_y, "vert-origin-y")
HTML5_ATOM(v_ideographic, "v-ideographic")
HTML5_ATOM(preservealpha, "preservealpha")
HTML5_ATOM(preserveAlpha, "preserveAlpha")
HTML5_ATOM(scriptminsize, "scriptminsize")
HTML5_ATOM(specification, "specification")
HTML5_ATOM(xlink_actuate, "xlink:actuate")
HTML5_ATOM(actuate, "actuate")
HTML5_ATOM(xlink_arcrole, "xlink:arcrole")
HTML5_ATOM(arcrole, "arcrole")
HTML5_ATOM(accept_charset, "accept-charset")
HTML5_ATOM(alignmentscope, "alignmentscope")
HTML5_ATOM(aria_multiline, "aria-multiline")
HTML5_ATOM(baseline_shift, "baseline-shift")
HTML5_ATOM(horiz_origin_x, "horiz-origin-x")
HTML5_ATOM(horiz_origin_y, "horiz-origin-y")
HTML5_ATOM(onbeforeupdate, "onbeforeupdate")
HTML5_ATOM(onfilterchange, "onfilterchange")
HTML5_ATOM(onrowsinserted, "onrowsinserted")
HTML5_ATOM(onbeforeunload, "onbeforeunload")
HTML5_ATOM(mathbackground, "mathbackground")
HTML5_ATOM(letter_spacing, "letter-spacing")
HTML5_ATOM(lighting_color, "lighting-color")
HTML5_ATOM(thickmathspace, "thickmathspace")
HTML5_ATOM(text_rendering, "text-rendering")
HTML5_ATOM(v_mathematical, "v-mathematical")
HTML5_ATOM(pointer_events, "pointer-events")
HTML5_ATOM(primitiveunits, "primitiveunits")
HTML5_ATOM(primitiveUnits, "primitiveUnits")
HTML5_ATOM(referrerpolicy, "referrerpolicy")
HTML5_ATOM(systemlanguage, "systemlanguage")
HTML5_ATOM(systemLanguage, "systemLanguage")
HTML5_ATOM(stroke_linecap, "stroke-linecap")
HTML5_ATOM(subscriptshift, "subscriptshift")
HTML5_ATOM(stroke_opacity, "stroke-opacity")
HTML5_ATOM(aria_dropeffect, "aria-dropeffect")
HTML5_ATOM(aria_labelledby, "aria-labelledby")
HTML5_ATOM(aria_templateid, "aria-templateid")
HTML5_ATOM(color_rendering, "color-rendering")
HTML5_ATOM(contenteditable, "contenteditable")
HTML5_ATOM(diffuseconstant, "diffuseconstant")
HTML5_ATOM(diffuseConstant, "diffuseConstant")
HTML5_ATOM(ondataavailable, "ondataavailable")
HTML5_ATOM(oncontrolselect, "oncontrolselect")
HTML5_ATOM(image_rendering, "image-rendering")
HTML5_ATOM(mediummathspace, "mediummathspace")
HTML5_ATOM(text_decoration, "text-decoration")
HTML5_ATOM(shape_rendering, "shape-rendering")
HTML5_ATOM(stroke_linejoin, "stroke-linejoin")
HTML5_ATOM(repeat_template, "repeat-template")
HTML5_ATOM(aria_describedby, "aria-describedby")
HTML5_ATOM(font_size_adjust, "font-size-adjust")
HTML5_ATOM(kernelunitlength, "kernelunitlength")
HTML5_ATOM(kernelUnitLength, "kernelUnitLength")
HTML5_ATOM(onbeforeactivate, "onbeforeactivate")
HTML5_ATOM(onpropertychange, "onpropertychange")
HTML5_ATOM(ondatasetchanged, "ondatasetchanged")
HTML5_ATOM(maskcontentunits, "maskcontentunits")
HTML5_ATOM(maskContentUnits, "maskContentUnits")
HTML5_ATOM(patterntransform, "patterntransform")
HTML5_ATOM(patternTransform, "patternTransform")
HTML5_ATOM(requiredfeatures, "requiredfeatures")
HTML5_ATOM(requiredFeatures, "requiredFeatures")
HTML5_ATOM(rendering_intent, "rendering-intent")
HTML5_ATOM(specularexponent, "specularexponent")
HTML5_ATOM(specularExponent, "specularExponent")
HTML5_ATOM(specularconstant, "specularconstant")
HTML5_ATOM(specularConstant, "specularConstant")
HTML5_ATOM(superscriptshift, "superscriptshift")
HTML5_ATOM(stroke_dasharray, "stroke-dasharray")
HTML5_ATOM(xchannelselector, "xchannelselector")
HTML5_ATOM(xChannelSelector, "xChannelSelector")
HTML5_ATOM(ychannelselector, "ychannelselector")
HTML5_ATOM(yChannelSelector, "yChannelSelector")
HTML5_ATOM(aria_autocomplete, "aria-autocomplete")
HTML5_ATOM(enable_background, "enable-background")
HTML5_ATOM(dominant_baseline, "dominant-baseline")
HTML5_ATOM(gradienttransform, "gradienttransform")
HTML5_ATOM(gradientTransform, "gradientTransform")
HTML5_ATOM(onbefordeactivate, "onbefordeactivate")
HTML5_ATOM(ondatasetcomplete, "ondatasetcomplete")
HTML5_ATOM(overline_position, "overline-position")
HTML5_ATOM(onbeforeeditfocus, "onbeforeeditfocus")
HTML5_ATOM(limitingconeangle, "limitingconeangle")
HTML5_ATOM(limitingConeAngle, "limitingConeAngle")
HTML5_ATOM(verythinmathspace, "verythinmathspace")
HTML5_ATOM(stroke_dashoffset, "stroke-dashoffset")
HTML5_ATOM(stroke_miterlimit, "stroke-miterlimit")
HTML5_ATOM(alignment_baseline, "alignment-baseline")
HTML5_ATOM(onreadystatechange, "onreadystatechange")
HTML5_ATOM(overline_thickness, "overline-thickness")
HTML5_ATOM(underline_position, "underline-position")
HTML5_ATOM(verythickmathspace, "verythickmathspace")
HTML5_ATOM(requiredextensions, "requiredextensions")
HTML5_ATOM(requiredExtensions, "requiredExtensions")
HTML5_ATOM(color_interpolation, "color-interpolation")
HTML5_ATOM(underline_thickness, "underline-thickness")
HTML5_ATOM(preserveaspectratio, "preserveaspectratio")
HTML5_ATOM(preserveAspectRatio, "preserveAspectRatio")
HTML5_ATOM(patterncontentunits, "patterncontentunits")
HTML5_ATOM(patternContentUnits, "patternContentUnits")
HTML5_ATOM(aria_multiselectable, "aria-multiselectable")
HTML5_ATOM(scriptsizemultiplier, "scriptsizemultiplier")
HTML5_ATOM(aria_activedescendant, "aria-activedescendant")
HTML5_ATOM(veryverythinmathspace, "veryverythinmathspace")
HTML5_ATOM(veryverythickmathspace, "veryverythickmathspace")
HTML5_ATOM(strikethrough_position, "strikethrough-position")
HTML5_ATOM(strikethrough_thickness, "strikethrough-thickness")
HTML5_ATOM(glyph_orientation_vertical, "glyph-orientation-vertical")
HTML5_ATOM(color_interpolation_filters, "color-interpolation-filters")
HTML5_ATOM(glyph_orientation_horizontal, "glyph-orientation-horizontal")
HTML5_ATOM(b, "b")
HTML5_ATOM(g, "g")
HTML5_ATOM(i, "i")
HTML5_ATOM(q, "q")
HTML5_ATOM(s, "s")
HTML5_ATOM(u, "u")
HTML5_ATOM(br, "br")
HTML5_ATOM(ci, "ci")
HTML5_ATOM(cn, "cn")
HTML5_ATOM(dd, "dd")
HTML5_ATOM(dl, "dl")
HTML5_ATOM(dt, "dt")
HTML5_ATOM(em, "em")
HTML5_ATOM(eq, "eq")
HTML5_ATOM(fn, "fn")
HTML5_ATOM(h1, "h1")
HTML5_ATOM(h2, "h2")
HTML5_ATOM(h3, "h3")
HTML5_ATOM(h4, "h4")
HTML5_ATOM(h5, "h5")
HTML5_ATOM(h6, "h6")
HTML5_ATOM(gt, "gt")
HTML5_ATOM(hr, "hr")
HTML5_ATOM(li, "li")
HTML5_ATOM(ln, "ln")
HTML5_ATOM(lt, "lt")
HTML5_ATOM(or_, "or")
HTML5_ATOM(pi, "pi")
HTML5_ATOM(rb, "rb")
HTML5_ATOM(rp, "rp")
HTML5_ATOM(tt, "tt")
HTML5_ATOM(and_, "and")
HTML5_ATOM(arg, "arg")
HTML5_ATOM(abs, "abs")
HTML5_ATOM(big, "big")
HTML5_ATOM(bdo, "bdo")
HTML5_ATOM(csc, "csc")
HTML5_ATOM(col, "col")
HTML5_ATOM(cos, "cos")
HTML5_ATOM(cot, "cot")
HTML5_ATOM(del, "del")
HTML5_ATOM(dfn, "dfn")
HTML5_ATOM(exp, "exp")
HTML5_ATOM(gcd, "gcd")
HTML5_ATOM(geq, "geq")
HTML5_ATOM(img, "img")
HTML5_ATOM(ins, "ins")
HTML5_ATOM(int_, "int")
HTML5_ATOM(kbd, "kbd")
HTML5_ATOM(log, "log")
HTML5_ATOM(lcm, "lcm")
HTML5_ATOM(leq, "leq")
HTML5_ATOM(mtd, "mtd")
HTML5_ATOM(map, "map")
HTML5_ATOM(mtr, "mtr")
HTML5_ATOM(neq, "neq")
HTML5_ATOM(not_, "not")
HTML5_ATOM(nav, "nav")
HTML5_ATOM(pre, "pre")
HTML5_ATOM(rem, "rem")
HTML5_ATOM(sub, "sub")
HTML5_ATOM(sec, "sec")
HTML5_ATOM(sum, "sum")
HTML5_ATOM(sin, "sin")
HTML5_ATOM(sep, "sep")
HTML5_ATOM(sup, "sup")
HTML5_ATOM(set, "set")
HTML5_ATOM(tan, "tan")
HTML5_ATOM(use, "use")
HTML5_ATOM(var, "var")
HTML5_ATOM(wbr, "wbr")
HTML5_ATOM(xor_, "xor")
HTML5_ATOM(area, "area")
HTML5_ATOM(bvar, "bvar")
HTML5_ATOM(card, "card")
HTML5_ATOM(csch, "csch")
HTML5_ATOM(cosh, "cosh")
HTML5_ATOM(coth, "coth")
HTML5_ATOM(curl, "curl")
HTML5_ATOM(diff, "diff")
HTML5_ATOM(defs, "defs")
HTML5_ATOM(font, "font")
HTML5_ATOM(grad, "grad")
HTML5_ATOM(line, "line")
HTML5_ATOM(meta, "meta")
HTML5_ATOM(msub, "msub")
HTML5_ATOM(math, "math")
HTML5_ATOM(mark, "mark")
HTML5_ATOM(mean, "mean")
HTML5_ATOM(main, "main")
HTML5_ATOM(msup, "msup")
HTML5_ATOM(menu, "menu")
HTML5_ATOM(mrow, "mrow")
HTML5_ATOM(none, "none")
HTML5_ATOM(nest, "nest")
HTML5_ATOM(plus, "plus")
HTML5_ATOM(rule, "rule")
HTML5_ATOM(real, "real")
HTML5_ATOM(reln, "reln")
HTML5_ATOM(rect, "rect")
HTML5_ATOM(root, "root")
HTML5_ATOM(sech, "sech")
HTML5_ATOM(sinh, "sinh")
HTML5_ATOM(samp, "samp")
HTML5_ATOM(stop, "stop")
HTML5_ATOM(sdev, "sdev")
HTML5_ATOM(time, "time")
HTML5_ATOM(true_, "true")
HTML5_ATOM(tref, "tref")
HTML5_ATOM(tanh, "tanh")
HTML5_ATOM(view, "view")
HTML5_ATOM(aside, "aside")
HTML5_ATOM(audio, "audio")
HTML5_ATOM(apply, "apply")
HTML5_ATOM(embed, "embed")
HTML5_ATOM(false_, "false")
HTML5_ATOM(floor, "floor")
HTML5_ATOM(glyph, "glyph")
HTML5_ATOM(hkern, "hkern")
HTML5_ATOM(image, "image")
HTML5_ATOM(ident, "ident")
HTML5_ATOM(limit, "limit")
HTML5_ATOM(mfrac, "mfrac")
HTML5_ATOM(mpath, "mpath")
HTML5_ATOM(meter, "meter")
HTML5_ATOM(mover, "mover")
HTML5_ATOM(minus, "minus")
HTML5_ATOM(mroot, "mroot")
HTML5_ATOM(msqrt, "msqrt")
HTML5_ATOM(notin, "notin")
HTML5_ATOM(piece, "piece")
HTML5_ATOM(param, "param")
HTML5_ATOM(power, "power")
HTML5_ATOM(reals, "reals")
HTML5_ATOM(small_, "small")
HTML5_ATOM(track, "track")
HTML5_ATOM(tspan, "tspan")
HTML5_ATOM(times, "times")
HTML5_ATOM(union_, "union")
HTML5_ATOM(vkern, "vkern")
HTML5_ATOM(video, "video")
HTML5_ATOM(arcsec, "arcsec")
HTML5_ATOM(arccsc, "arccsc")
HTML5_ATOM(arctan, "arctan")
HTML5_ATOM(arcsin, "arcsin")
HTML5_ATOM(arccos, "arccos")
HTML5_ATOM(applet, "applet")
HTML5_ATOM(arccot, "arccot")
HTML5_ATOM(approx, "approx")
HTML5_ATOM(circle, "circle")
HTML5_ATOM(center, "center")
HTML5_ATOM(canvas, "canvas")
HTML5_ATOM(divide, "divide")
HTML5_ATOM(degree, "degree")
HTML5_ATOM(domain, "domain")
HTML5_ATOM(exists, "exists")
HTML5_ATOM(fetile, "fetile")
HTML5_ATOM(feTile, "feTile")
HTML5_ATOM(figure, "figure")
HTML5_ATOM(forall, "forall")
HTML5_ATOM(footer, "footer")
HTML5_ATOM(hgroup, "hgroup")
HTML5_ATOM(header, "header")
HTML5_ATOM(keygen, "keygen")
HTML5_ATOM(lambda, "lambda")
HTML5_ATOM(legend, "legend")
HTML5_ATOM(mspace, "mspace")
HTML5_ATOM(mtable, "mtable")
HTML5_ATOM(mstyle, "mstyle")
HTML5_ATOM(mglyph, "mglyph")
HTML5_ATOM(median, "median")
HTML5_ATOM(munder, "munder")
HTML5_ATOM(marker, "marker")
HTML5_ATOM(merror, "merror")
HTML5_ATOM(moment, "moment")
HTML5_ATOM(matrix, "matrix")
HTML5_ATOM(output, "output")
HTML5_ATOM(primes, "primes")
HTML5_ATOM(source, "source")
HTML5_ATOM(strike, "strike")
HTML5_ATOM(strong, "strong")
HTML5_ATOM(switch_, "switch")
HTML5_ATOM(symbol, "symbol")
HTML5_ATOM(subset, "subset")
HTML5_ATOM(tbreak, "tbreak")
HTML5_ATOM(vector, "vector")
HTML5_ATOM(article, "article")
HTML5_ATOM(animate, "animate")
HTML5_ATOM(arcsech, "arcsech")
HTML5_ATOM(arccsch, "arccsch")
HTML5_ATOM(arctanh, "arctanh")
HTML5_ATOM(arcsinh, "arcsinh")
HTML5_ATOM(arccosh, "arccosh")
HTML5_ATOM(arccoth, "arccoth")
HTML5_ATOM(acronym, "acronym")
HTML5_ATOM(bgsound, "bgsound")
HTML5_ATOM(compose, "compose")
HTML5_ATOM(ceiling, "ceiling")
HTML5_ATOM(csymbol, "csymbol")
HTML5_ATOM(discard, "discard")
HTML5_ATOM(details, "details")
HTML5_ATOM(ellipse, "ellipse")
HTML5_ATOM(fefunca, "fefunca")
HTML5_ATOM(feFuncA, "feFuncA")
HTML5_ATOM(fefuncb, "fefuncb")
HTML5_ATOM(feFuncB, "feFuncB")
HTML5_ATOM(feblend, "feblend")
HTML5_ATOM(feBlend, "feBlend")
HTML5_ATOM(feflood, "feflood")
HTML5_ATOM(feFlood, "feFlood")
HTML5_ATOM(feimage, "feimage")
HTML5_ATOM(feImage, "feImage")
HTML5_ATOM(femerge, "femerge")
HTML5_ATOM(feMerge, "feMerge")
HTML5_ATOM(fefuncg, "fefuncg")
HTML5_ATOM(feFuncG, "feFuncG")
HTML5_ATOM(fefuncr, "fefuncr")
HTML5_ATOM(feFuncR, "feFuncR")
HTML5_ATOM(handler, "handler")
HTML5_ATOM(inverse, "inverse")
HTML5_ATOM(implies, "implies")
HTML5_ATOM(isindex, "isindex")
HTML5_ATOM(logbase, "logbase")
HTML5_ATOM(listing, "listing")
HTML5_ATOM(mfenced, "mfenced")
HTML5_ATOM(mpadded, "mpadded")
HTML5_ATOM(marquee, "marquee")
HTML5_ATOM(maction, "maction")
HTML5_ATOM(msubsup, "msubsup")
HTML5_ATOM(polygon, "polygon")
HTML5_ATOM(picture, "picture")
HTML5_ATOM(product, "product")
HTML5_ATOM(setdiff, "setdiff")
HTML5_ATOM(section, "section")
HTML5_ATOM(tendsto, "tendsto")
HTML5_ATOM(uplimit, "uplimit")
HTML5_ATOM(altglyph, "altglyph")
HTML5_ATOM(altGlyph, "altGlyph")
HTML5_ATOM(basefont, "basefont")
HTML5_ATOM(clippath, "clippath")
HTML5_ATOM(clipPath, "clipPath")
HTML5_ATOM(codomain, "codomain")
HTML5_ATOM(emptyset, "emptyset")
HTML5_ATOM(factorof, "factorof")
HTML5_ATOM(fieldset, "fieldset")
HTML5_ATOM(feoffset, "feoffset")
HTML5_ATOM(feOffset, "feOffset")
HTML5_ATOM(interval, "interval")
HTML5_ATOM(integers, "integers")
HTML5_ATOM(infinity, "infinity")
HTML5_ATOM(listener, "listener")
HTML5_ATOM(lowlimit, "lowlimit")
HTML5_ATOM(metadata, "metadata")
HTML5_ATOM(menclose, "menclose")
HTML5_ATOM(menuitem, "menuitem")
HTML5_ATOM(mphantom, "mphantom")
HTML5_ATOM(polyline, "polyline")
HTML5_ATOM(prefetch, "prefetch")
HTML5_ATOM(progress, "progress")
HTML5_ATOM(prsubset, "prsubset")
HTML5_ATOM(quotient, "quotient")
HTML5_ATOM(selector, "selector")
HTML5_ATOM(textpath, "textpath")
HTML5_ATOM(textPath, "textPath")
HTML5_ATOM(variance, "variance")
HTML5_ATOM(animation, "animation")
HTML5_ATOM(conjugate, "conjugate")
HTML5_ATOM(condition, "condition")
HTML5_ATOM(complexes, "complexes")
HTML5_ATOM(font_face, "font-face")
HTML5_ATOM(factorial, "factorial")
HTML5_ATOM(intersect, "intersect")
HTML5_ATOM(imaginary, "imaginary")
HTML5_ATOM(laplacian, "laplacian")
HTML5_ATOM(matrixrow, "matrixrow")
HTML5_ATOM(notsubset, "notsubset")
HTML5_ATOM(otherwise, "otherwise")
HTML5_ATOM(piecewise, "piecewise")
HTML5_ATOM(rationals, "rationals")
HTML5_ATOM(semantics, "semantics")
HTML5_ATOM(transpose, "transpose")
HTML5_ATOM(annotation, "annotation")
HTML5_ATOM(blockquote, "blockquote")
HTML5_ATOM(divergence, "divergence")
HTML5_ATOM(eulergamma, "eulergamma")
HTML5_ATOM(equivalent, "equivalent")
HTML5_ATOM(figcaption, "figcaption")
HTML5_ATOM(imaginaryi, "imaginaryi")
HTML5_ATOM(malignmark, "malignmark")
HTML5_ATOM(munderover, "munderover")
HTML5_ATOM(mlabeledtr, "mlabeledtr")
HTML5_ATOM(notanumber, "notanumber")
HTML5_ATOM(solidcolor, "solidcolor")
HTML5_ATOM(altglyphdef, "altglyphdef")
HTML5_ATOM(altGlyphDef, "altGlyphDef")
HTML5_ATOM(determinant, "determinant")
HTML5_ATOM(femergenode, "femergenode")
HTML5_ATOM(feMergeNode, "feMergeNode")
HTML5_ATOM(fecomposite, "fecomposite")
HTML5_ATOM(feComposite, "feComposite")
HTML5_ATOM(fespotlight, "fespotlight")
HTML5_ATOM(feSpotLight, "feSpotLight")
HTML5_ATOM(maligngroup, "maligngroup")
HTML5_ATOM(mprescripts, "mprescripts")
HTML5_ATOM(momentabout, "momentabout")
HTML5_ATOM(notprsubset, "notprsubset")
HTML5_ATOM(partialdiff, "partialdiff")
HTML5_ATOM(altglyphitem, "altglyphitem")
HTML5_ATOM(altGlyphItem, "altGlyphItem")
HTML5_ATOM(animatecolor, "animatecolor")
HTML5_ATOM(animateColor, "animateColor")
HTML5_ATOM(datatemplate, "datatemplate")
HTML5_ATOM(exponentiale, "exponentiale")
HTML5_ATOM(feturbulence, "feturbulence")
HTML5_ATOM(feTurbulence, "feTurbulence")
HTML5_ATOM(fepointlight, "fepointlight")
HTML5_ATOM(fePointLight, "fePointLight")
HTML5_ATOM(fedropshadow, "fedropshadow")
HTML5_ATOM(feDropShadow, "feDropShadow")
HTML5_ATOM(femorphology, "femorphology")
HTML5_ATOM(feMorphology, "feMorphology")
HTML5_ATOM(outerproduct, "outerproduct")
HTML5_ATOM(animatemotion, "animatemotion")
HTML5_ATOM(animateMotion, "animateMotion")
HTML5_ATOM(font_face_src, "font-face-src")
HTML5_ATOM(font_face_uri, "font-face-uri")
HTML5_ATOM(foreignobject, "foreignobject")
HTML5_ATOM(fecolormatrix, "fecolormatrix")
HTML5_ATOM(feColorMatrix, "feColorMatrix")
HTML5_ATOM(missing_glyph, "missing-glyph")
HTML5_ATOM(mmultiscripts, "mmultiscripts")
HTML5_ATOM(scalarproduct, "scalarproduct")
HTML5_ATOM(vectorproduct, "vectorproduct")
HTML5_ATOM(definition_src, "definition-src")
HTML5_ATOM(font_face_name, "font-face-name")
HTML5_ATOM(fegaussianblur, "fegaussianblur")
HTML5_ATOM(feGaussianBlur, "feGaussianBlur")
HTML5_ATOM(fedistantlight, "fedistantlight")
HTML5_ATOM(feDistantLight, "feDistantLight")
HTML5_ATOM(lineargradient, "lineargradient")
HTML5_ATOM(linearGradient, "linearGradient")
HTML5_ATOM(naturalnumbers, "naturalnumbers")
HTML5_ATOM(radialgradient, "radialgradient")
HTML5_ATOM(radialGradient, "radialGradient")
HTML5_ATOM(animatetransform, "animatetransform")
HTML5_ATOM(animateTransform, "animateTransform")
HTML5_ATOM(cartesianproduct, "cartesianproduct")
HTML5_ATOM(font_face_format, "font-face-format")
HTML5_ATOM(feconvolvematrix, "feconvolvematrix")
HTML5_ATOM(feConvolveMatrix, "feConvolveMatrix")
HTML5_ATOM(fediffuselighting, "fediffuselighting")
HTML5_ATOM(feDiffuseLighting, "feDiffuseLighting")
HTML5_ATOM(fedisplacementmap, "fedisplacementmap")
HTML5_ATOM(feDisplacementMap, "feDisplacementMap")
HTML5_ATOM(fespecularlighting, "fespecularlighting")
HTML5_ATOM(feSpecularLighting, "feSpecularLighting")
HTML5_ATOM(domainofapplication, "domainofapplication")
HTML5_ATOM(fecomponenttransfer, "fecomponenttransfer")
HTML5_ATOM(feComponentTransfer, "feComponentTransfer")
