/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

interface nsIURI;

/**
 * This service maps file system paths where add-ons reside to the ID
 * of the add-on. Paths are added by the add-on manager. They can
 * looked up by anyone.
 */
[scriptable, uuid(fcd9e270-dfb1-11e3-8b68-0800200c9a66)]
interface amIAddonPathService : nsISupports
{
  /**
   * Given a path to a file, return the ID of the add-on that the file belongs
   * to. Returns an empty string if there is no add-on there. Note that if an
   * add-on is located at /a/b/c, then looking up the path /a/b/c/d will return
   * that add-on.
   */
  AString findAddonId(in AString path);

  /**
   * Call this function to inform the service that the given file system path is
   * associated with the given add-on ID.
   */
  void insertPath(in AString path, in AString addonId);

  /**
   * Given a URI to a file, return the ID of the add-on that the file belongs
   * to. Returns an empty string if there is no add-on there.
   */
  AString mapURIToAddonId(in nsIURI aURI);
};
