GENERATED_EVENT(AddonEvent)
GENERATED_EVENT(AnimationPlaybackEvent)
GENERATED_EVENT(AutocompleteErrorEvent)
GENERATED_EVENT(BlobEvent)
GENERATED_EVENT(CaretStateChangedEvent)
GENERATED_EVENT(CloseEvent)
GENERATED_EVENT(DeviceLightEvent)
GENERATED_EVENT(DeviceOrientationEvent)
GENERATED_EVENT(DeviceProximityEvent)
GENERATED_EVENT(DeviceStorageAreaChangedEvent)
GENERATED_EVENT(DeviceStorageChangeEvent)
GENERATED_EVENT(DownloadEvent)
GENERATED_EVENT(ErrorEvent)
GENERATED_EVENT(FontFaceSetLoadEvent)
GENERATED_EVENT(GamepadAxisMoveEvent)
GENERATED_EVENT(GamepadButtonEvent)
GENERATED_EVENT(GamepadEvent)
GENERATED_EVENT(HashChangeEvent)
GENERATED_EVENT(HiddenPluginEvent)
GENERATED_EVENT(ImageCaptureErrorEvent)
GENERATED_EVENT(MediaStreamEvent)
GENERATED_EVENT(MediaStreamTrackEvent)
GENERATED_EVENT(MozSettingsEvent)
GENERATED_EVENT(MozSettingsTransactionEvent)
GENERATED_EVENT(PageTransitionEvent)
GENERATED_EVENT(PerformanceEntryEvent)
GENERATED_EVENT(PluginCrashedEvent)
GENERATED_EVENT(PopStateEvent)
GENERATED_EVENT(PopupBlockedEvent)
GENERATED_EVENT(PresentationConnectionAvailableEvent)
GENERATED_EVENT(PresentationConnectionCloseEvent)
GENERATED_EVENT(ProgressEvent)
GENERATED_EVENT(RTCDTMFToneChangeEvent)
GENERATED_EVENT(RTCDataChannelEvent)
GENERATED_EVENT(RTCPeerConnectionIceEvent)
GENERATED_EVENT(RTCTrackEvent)
GENERATED_EVENT(RecordErrorEvent)
GENERATED_EVENT(ScrollViewChangeEvent)
GENERATED_EVENT(ServiceWorkerMessageEvent)
GENERATED_EVENT(SpeechRecognitionEvent)
GENERATED_EVENT(SpeechSynthesisErrorEvent)
GENERATED_EVENT(SpeechSynthesisEvent)
GENERATED_EVENT(StyleRuleChangeEvent)
GENERATED_EVENT(StyleSheetApplicableStateChangeEvent)
GENERATED_EVENT(StyleSheetChangeEvent)
GENERATED_EVENT(TCPServerSocketEvent)
GENERATED_EVENT(TCPSocketErrorEvent)
GENERATED_EVENT(TCPSocketEvent)
GENERATED_EVENT(TrackEvent)
GENERATED_EVENT(UDPMessageEvent)
GENERATED_EVENT(UserProximityEvent)
GENERATED_EVENT(WebGLContextEvent)
