/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM ../../../dist/idl/nsIDOMHTMLPictureElement.idl
 */

#ifndef __gen_nsIDOMHTMLPictureElement_h__
#define __gen_nsIDOMHTMLPictureElement_h__


#ifndef __gen_nsIDOMHTMLElement_h__
#include "nsIDOMHTMLElement.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif

/* starting interface:    nsIDOMHTMLPictureElement */
#define NS_IDOMHTMLPICTUREELEMENT_IID_STR "e0e5ac7f-b969-494c-a61e-9d740e38abba"

#define NS_IDOMHTMLPICTUREELEMENT_IID \
  {0xe0e5ac7f, 0xb969, 0x494c, \
    { 0xa6, 0x1e, 0x9d, 0x74, 0x0e, 0x38, 0xab, 0xba }}

class NS_NO_VTABLE nsIDOMHTMLPictureElement : public nsISupports {
 public:

  NS_DECLARE_STATIC_IID_ACCESSOR(NS_IDOMHTMLPICTUREELEMENT_IID)

};

  NS_DEFINE_STATIC_IID_ACCESSOR(nsIDOMHTMLPictureElement, NS_IDOMHTMLPICTUREELEMENT_IID)

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIDOMHTMLPICTUREELEMENT \
  /* no methods! */

/* Use this macro when declaring the members of this interface when the
   class doesn't implement the interface. This is useful for forwarding. */
#define NS_DECL_NON_VIRTUAL_NSIDOMHTMLPICTUREELEMENT \
  /* no methods! */

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIDOMHTMLPICTUREELEMENT(_to) \
  /* no methods! */

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSIDOMHTMLPICTUREELEMENT(_to) \
  /* no methods! */

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsDOMHTMLPictureElement : public nsIDOMHTMLPictureElement
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIDOMHTMLPICTUREELEMENT

  nsDOMHTMLPictureElement();

private:
  ~nsDOMHTMLPictureElement();

protected:
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS(nsDOMHTMLPictureElement, nsIDOMHTMLPictureElement)

nsDOMHTMLPictureElement::nsDOMHTMLPictureElement()
{
  /* member initializers and constructor code */
}

nsDOMHTMLPictureElement::~nsDOMHTMLPictureElement()
{
  /* destructor code */
}

/* End of implementation class template. */
#endif


#endif /* __gen_nsIDOMHTMLPictureElement_h__ */
